﻿namespace Giraffe.Fixi

open Microsoft.AspNetCore.Http
open Microsoft.Extensions.Primitives

/// Module to hold extensions
[<AutoOpen>]
module Extensions =

    /// Extensions to the header dictionary
    type IHeaderDictionary with

        /// <summary>Indicates that the request was generated from fixi</summary>
        member this.IsFixiRequest
            with get () = not (this["FX-Request"] = StringValues.Empty)

/// <summary>Valid values for the <c>fx-swap</c> attribute</summary>
type FixiSwap =

    /// <summary>Replace the entire target element with the response (the default)</summary>
    | OuterHtml

    /// <summary>Replace the inner HTML of the target element</summary>
    | InnerHtml

    /// <summary>Insert the response before the target element</summary>
    | BeforeBegin

    /// <summary>Insert the response before the first child of the target element</summary>
    | AfterBegin

    /// <summary>Insert the response after the last child of the target element</summary>
    | BeforeEnd

    /// <summary>Insert the response after the target element</summary>
    | AfterEnd

    override this.ToString() =
        match this with
        | OuterHtml   -> "outerHTML"
        | InnerHtml   -> "innerHTML"
        | BeforeBegin -> "beforebegin"
        | AfterBegin  -> "afterbegin"
        | BeforeEnd   -> "beforeend"
        | AfterEnd    -> "afterend"


open Giraffe.ViewEngine

/// <summary>Attributes and flags for fixi</summary>
[<AutoOpen>]
module FixiAttrs =

    /// <summary>Issue a request to the given URL</summary>
    /// <param name="url">The URL to which the request should be issued</param>
    /// <returns>A configured <c>fx-action</c> attribute</returns>
    let _fxAction url =
        attr "fx-action" url

    /// <summary>Exclude this element (and below) from fixi's behavior</summary>
    let _fxIgnore =
        flag "fx-ignore"

    /// <summary>Specify the HTTP method for the fixi request</summary>
    /// <param name="it">The HTTP method to use for the request</param>
    /// <returns>A configured <c>fx-method</c> attribute</returns>
    /// <remarks>Consider <c>_fxGet</c>, <c>_fxPost</c>, etc.</remarks>
    let _fxMethod (it: string) =
        attr "fx-method" (it.ToUpperInvariant())

    /// <summary>Specify the swap strategy</summary>
    /// <param name="swap">The swap strategy to use</param>
    /// <returns>A configured <c>fx-swap</c> attribute</returns>
    let _fxSwap (swap: FixiSwap) =
        attr "fx-swap" (string swap)

    /// <summary>Specify the target for the swap</summary>
    /// <param name="selector">The CSS selector for the swap target</param>
    /// <returns>A configured <c>fx-target</c> attribute</returns>
    let _fxTarget selector =
        attr "fx-target" selector

    /// <summary>The event which should trigger this request</summary>
    /// <param name="event">The name of the event</param>
    /// <returns>A configured <c>fx-trigger</c> attribute</returns>
    /// <remarks>The event should not start with "on"; use "click", not "onclick"</remarks>
    let _fxTrigger event =
        attr "fx-trigger" event

/// <summary>Canned <c>fx-method</c> attributes for common HTTP methods</summary>
[<AutoOpen>]
module FixiMethods =

    /// <summary>An <c>fx-method="DELETE"</c> attribute</summary>
    let _fxDelete =
        _fxMethod "DELETE"

    /// <summary>An <c>fx-method="HEAD"</c> attribute</summary>
    let _fxHead =
        _fxMethod "HEAD"

    /// <summary>An <c>fx-method="GET"</c> attribute</summary>
    let _fxGet =
        _fxMethod "GET"

    /// <summary>An <c>fx-method="PATCH"</c> attribute</summary>
    let _fxPatch =
        _fxMethod "PATCH"

    /// <summary>An <c>fx-method="POST"</c> attribute</summary>
    let _fxPost =
        _fxMethod "POST"

    /// <summary>An <c>fx-method="PUT"</c> attribute</summary>
    let _fxPut =
        _fxMethod "PUT"
