﻿/// Unit tests for Giraffe.Fixi
module FixiTests

open Expecto
open Giraffe.ViewEngine

/// Custom expectation for this test
module Expect =
    /// Expect the rendered node to match the given HTML markup text
    let html node markup =
        Expect.equal (RenderView.AsString.htmlNode node) markup "Rendered HTML incorrect"

open Giraffe.Fixi

/// Tests for the FixiSwap type
let swap =
    testList "FixiSwap" [
        test "OuterHtml is correct" {
            Expect.equal (string FixiSwap.OuterHtml) "outerHTML" "Outer HTML swap value incorrect"
        }
        test "InnerHtml is correct" {
            Expect.equal (string FixiSwap.InnerHtml) "innerHTML" "Inner HTML swap value incorrect"
        }
        test "BeforeBegin is correct" {
            Expect.equal (string FixiSwap.BeforeBegin) "beforebegin" "Before Begin swap value incorrect"
        }
        test "BeforeEnd is correct" {
            Expect.equal (string FixiSwap.BeforeEnd) "beforeend" "Before End swap value incorrect"
        }
        test "AfterBegin is correct" {
            Expect.equal (string FixiSwap.AfterBegin) "afterbegin" "After Begin swap value incorrect"
        }
        test "AfterEnd is correct" {
            Expect.equal (string FixiSwap.AfterEnd) "afterend" "After End swap value incorrect"
        }
    ]


/// Tests for the FixiAttrs module
let attributes =
    testList "FixiAttrs" [
        test "_fxAction succeeds" {
            Expect.html (div [ _fxAction "/do-this" ] []) """<div fx-action="/do-this"></div>"""
        }
        test "_fxIgnore succeeds" {
            Expect.html (span [ _fxIgnore ] [ rawText "this" ]) """<span fx-ignore>this</span>"""
        }
        test "_fxMethod succeeds" {
            Expect.html (button [ _fxMethod "options" ] []) """<button fx-method="OPTIONS"></button>"""
        }
        test "_fxSwap succeeds" {
            Expect.html (p [ _fxSwap FixiSwap.InnerHtml ] []) """<p fx-swap="innerHTML"></p>"""
        }
        test "_fxTarget succeeds" {
            Expect.html (button [ _fxTarget "#target" ] []) """<button fx-target="#target"></button>"""
        }
        test "_fxTrigger succeeds" {
            Expect.html (strong [ _fxTrigger "blur" ] []) """<strong fx-trigger="blur"></strong>"""
        }
    ]

/// Tests for the FixiMethods module
let methods =
    testList "FixiMethods" [
        test "_fxDelete succeeds" {
            match _fxDelete with
            | XmlAttribute.KeyValue(attr, value) ->
                Expect.equal attr "fx-method" "The attribute key is incorrect"
                Expect.equal value "DELETE" "The attribute value is incorrect"
            | XmlAttribute.Boolean _ -> Expect.isTrue false "The _fxDelete attribute is incorrect"
        }
        test "_fxHead succeeds" {
            match _fxHead with
            | XmlAttribute.KeyValue(attr, value) ->
                Expect.equal attr "fx-method" "The attribute key is incorrect"
                Expect.equal value "HEAD" "The attribute value is incorrect"
            | XmlAttribute.Boolean _ -> Expect.isTrue false "The _fxHead attribute is incorrect"
        }
        test "_fxGet succeeds" {
            match _fxGet with
            | XmlAttribute.KeyValue(attr, value) ->
                Expect.equal attr "fx-method" "The attribute key is incorrect"
                Expect.equal value "GET" "The attribute value is incorrect"
            | XmlAttribute.Boolean _ -> Expect.isTrue false "The _fxGet attribute is incorrect"
        }
        test "_fxPatch succeeds" {
            match _fxPatch with
            | XmlAttribute.KeyValue(attr, value) ->
                Expect.equal attr "fx-method" "The attribute key is incorrect"
                Expect.equal value "PATCH" "The attribute value is incorrect"
            | XmlAttribute.Boolean _ -> Expect.isTrue false "The _fxPatch attribute is incorrect"
        }
        test "_fxPost succeeds" {
            match _fxPost with
            | XmlAttribute.KeyValue(attr, value) ->
                Expect.equal attr "fx-method" "The attribute key is incorrect"
                Expect.equal value "POST" "The attribute value is incorrect"
            | XmlAttribute.Boolean _ -> Expect.isTrue false "The _fxPost attribute is incorrect"
        }
        test "_fxPut succeeds" {
            match _fxPut with
            | XmlAttribute.KeyValue(attr, value) ->
                Expect.equal attr "fx-method" "The attribute key is incorrect"
                Expect.equal value "PUT" "The attribute value is incorrect"
            | XmlAttribute.Boolean _ -> Expect.isTrue false "The _fxPut attribute is incorrect"
        }
    ]

let allTests = testList "Giraffe.Fixi" [ swap; attributes; methods ]

[<EntryPoint>]
let main args = runTestsWithCLIArgs [] args allTests
